﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;

namespace ImageViewer
{
    public partial class MainForm : Form
    {
        private static readonly int IMAGES_COLUMNS = 5;


        private Core _core;

        public string[] CurrentImages;

        private bool _mouseDown = false;

        private string _startDir = null;

        private string _startImage = null;

        private bool _loadingImages = false;


        public MainForm(string arg)
        {
            InitializeComponent();

            _core = new Core();
            if (arg != null)
            {
                if (!Path.HasExtension(arg))
                    _startDir = arg;
                else if (Core.IMAGE_EXTENSIONS.Contains(Path.GetExtension(arg)))
                {
                    _startDir = Path.GetFullPath(arg);
                    _startImage = arg;
                }
            }
        }

        #region FormActions

        private void MainForm_Shown(object sender, EventArgs e)
        {
            FillDrives();
            try
            {
                if (_startDir != null)
                {
                    string[] dirs = _startDir.Split(@"\".ToCharArray());
                    TreeNode node = tvDirectory.Nodes.Find(dirs[0], true)[0];
                    for (int i = 1; i < dirs.Length; i++)
                    {
                        LoadNodeData(node);
                        node = tvDirectory.Nodes.Find(dirs[i], true)[0];
                    }
                    LoadImages(node);
                }
                if (_startImage != null)
                {
                    ImageView frm = new ImageView(this, _startImage, new Bitmap(_startImage), "", 0);
                    frm.ShowDialog();
                    frm.Dispose();
                }
            }
            catch (Exception ex)
            {
            }
        }

        #endregion

        #region TreeView actions

        private void tvDirectory_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (!_loadingImages)
            {
                if (e.Action != TreeViewAction.Collapse && e.Action != TreeViewAction.Expand && _mouseDown)
                {
                    _mouseDown = false;
                    LoadNodeData(e.Node);
                    LoadImages(e.Node);
                }
            }
        }

        #endregion

        private void FillDrives()
        {
            // Show logical drives
            string[] drives = _core.GetLogicalDrives();
            TreeNode[] driveNodes = new TreeNode[drives.Length];
            for(int i = 0; i < drives.Length; i++)
            {
                driveNodes[i] = tvDirectory.Nodes.Add(drives[i]);
            }
            tvDirectory.ExpandAll();
        }

        private void LoadNodeData(TreeNode node)
        {
            string[] dirs = _core.GetDirectoriesIn(node.FullPath);
            if (dirs == null) return;
            foreach (string dir in dirs)
            {
                node.Nodes.Add(dir);
            }
            node.Expand();
        }

        private void LoadImages(TreeNode node)
        {
            _loadingImages = true;

            string[] images = _core.GetImagesIn(node.FullPath);
            CurrentImages = images;
            if (images == null)
            {
                _loadingImages = false;
                return;
            }

            // Prepare layout table
            tlpImagesTable.Controls.Clear();
            tlpImagesTable.ColumnStyles.Clear();
            tlpImagesTable.RowStyles.Clear();
            tlpImagesTable.ColumnCount = IMAGES_COLUMNS;
            tlpImagesTable.RowCount = (int)Math.Round((float)images.Length / (float)IMAGES_COLUMNS) + 1;
            for(int a = 0; a < tlpImagesTable.RowCount; a++)
                tlpImagesTable.RowStyles.Add(new RowStyle(SizeType.Percent, tlpImagesTable.ClientSize.Height / tlpImagesTable.RowCount));
            for(int b = 0; b < tlpImagesTable.ColumnCount; b++)
                tlpImagesTable.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, tlpImagesTable.ClientSize.Width / tlpImagesTable.ColumnCount));

            // Enter Images
            int i = 0;
            for (int column = 0; column < tlpImagesTable.ColumnCount; column++)
            {
                for(int row = 0; row < tlpImagesTable.RowCount; row++)
                {
                    if (i >= images.Length) break;
                    tlpImagesTable.Controls.Add(new ImagePreview(this, images[i], i));
                    Application.DoEvents();
                    i++;
                }
            }

            _loadingImages = false;
        }

        public Image GetImage(int index)
        {
            if (index >= tlpImagesTable.Controls.Count)
                return null;
            return ((ImagePreview)tlpImagesTable.Controls[index]).Image;
        }

        private void MainForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
                this.Close();
        }

        private void tvDirectory_MouseDown(object sender, MouseEventArgs e)
        {
            _mouseDown = true;
        }

        private void tvDirectory_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                LoadNodeData(tvDirectory.SelectedNode);
                LoadImages(tvDirectory.SelectedNode);
            }
            else if (e.KeyCode == Keys.Escape)
                this.Close();
            else if (e.KeyCode == Keys.S)
            {
                List<Image> images = new List<Image>();
                foreach (Control c in this.tlpImagesTable.Controls)
                {
                    images.Add(((ImagePreview)c).Image);
                }
                SlideShow frm = new SlideShow(images.ToArray());
                frm.ShowDialog();
                frm.Dispose();
            }
        }


    }
}
